import { system, world } from "@minecraft/server"
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { harvestDynamicStructure, updateDynamicData, writeDynamicData, fetchDynamicData, VFX } from "./functions.js"

// Este código não utiliza formas mais complexas, porque não é preciso.

const dynamicID = "harvest:"

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('ext:auto_harvest_component', {
        onTick: autoHarvestTick,
        beforeOnPlayerPlace: autoHarvestPlace,
        onPlayerInteract: autoHarvestMenu,
        onPlayerDestroy: autoHarvestDestroy
    })
});

function autoHarvestTick(event) {
    const block = event.block
    world.getDynamicPropertyIds().forEach(objDynamic => {
        if (objDynamic.startsWith(dynamicID)) {
            const harvestLocation = objDynamic.split(dynamicID)[1]
            const getData = fetchDynamicData(objDynamic)
            const Dimension = world.getDimension(block.dimension.id)
            if (harvestLocation == JSON.stringify(block.location)) {
                if (getData[0].durability > 0) {
                    if (getData[0].switch) {
                        if (getData[0].lvl == 0) {
                            Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", 2));
                        } else if (getData[0].lvl == 1) {
                            Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", 3));
                        } else if (getData[0].lvl == 2) {
                            Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", 4));
                        } else if (getData[0].lvl == 3) {
                            Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", 5));
                        }
                        squareGetBlocks(block, blocksCollectList, getData[0].size, 1, 1)
                    } else {
                        Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", 0));
                    }
                } else {
                    Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", 1));
                }
            }
        }
    })
}

function autoHarvestDestroy(event) {
    world.getDynamicPropertyIds().forEach(objDynamic => {
        if (objDynamic.startsWith(dynamicID)) {
            const Dimension = world.getDimension(event.block.dimension.id)
            const dynamicData = fetchDynamicData(objDynamic)
            const harvestLocation = objDynamic.split(dynamicID)[1]
            if (harvestLocation == JSON.stringify(event.block.location)) {
                if (dynamicData[0].private) {
                    if (dynamicData[0].owner == event.player.name) {
                        world.setDynamicProperty(`${dynamicID}${JSON.stringify(event.block.location)}`, null)
                    } else {
                        Dimension.setBlockType(itemTeleporterLocation, "ext:auto_harvest")
                        event.player.sendMessage({ translate: "warning.auto_harvest.break_block" })
                    }
                } else {
                    world.setDynamicProperty(`${dynamicID}${JSON.stringify(event.block.location)}`, null)
                }
            }
        }
    })
}

function autoHarvestPlace(event) {
    const [player, block] = [event.player, event.block]
    const dynamicNameInput = `harvest:${JSON.stringify(event.block.location)}`
    updateDynamicData(dynamicNameInput, harvestDynamicStructure(player, block))
}

function autoHarvestMenu(event) {
    const player = event.player;
    const block = event.block;
    const Dimension = world.getDimension(player.dimension.id);
    let dynamicName = getDynamicNameInLocation(event.block.location);
    const dynamicData = fetchDynamicData(dynamicName);
    const ui_config = new ModalFormData();
    const ui_upgrade = new ActionFormData();
    const ui_menu = new ActionFormData();
    const durabilityBlock = dynamicData[0].durability < dynamicData[0].max_durability

    const textT_title = { rawtext: [{ translate: "gui.auto_harvest.title" }, { text: ` §5[${dynamicData[0].lvl}]` }] }
    const ui_menu_body = {
        rawtext: [
            { text: "\n" },
            { translate: "gui.auto_harvest.ui_menu.body.1" }, { text: `§e${dynamicData[0].durability} / ${dynamicData[0].max_durability}§r\n\n` },
            { translate: "gui.auto_harvest.ui_menu.body.2" },
            { text: "\n " }
        ]
    }

    function blockMenuUI() {
        ui_menu.title(textT_title)
        ui_menu.body(ui_menu_body)
        ui_menu.button({ translate: "gui.auto_harvest.ui_menu.button1" }, "textures/items/wood_hoe")
        if (durabilityBlock || dynamicData[0].lvl < 3) ui_menu.button({ translate: "gui.auto_harvest.ui_menu.button2" }, "textures/items/netherite_upgrade_smithing_template")
        ui_menu.show(player).then((data) => {
            const selectionButton = data.selection
            if (selectionButton == 0) {
                blockConfigUI()
            }
            if (selectionButton == 1) {
                blockUpgradeUI()
            }
        })
    }

    function blockConfigUI() {
        ui_config.title(textT_title);
        ui_config.toggle({ translate: "gui.auto_harvest.ui_config.button1" }, dynamicData[0].switch);
        ui_config.toggle({ translate: "gui.auto_harvest.ui_config.button2" }, dynamicData[0].private);

        ui_config.slider({ translate: "gui.auto_harvest.ui_config.slider1" }, 1, dynamicData[0].max_size, 1, dynamicData[0].size);

        ui_config.show(player).then((data) => {
            const [switchToggle, privateToggle, sizeSlider] = data.formValues;

            dynamicData[0].switch = switchToggle;
            dynamicData[0].private = privateToggle;
            dynamicData[0].size = sizeSlider;

            writeDynamicData(dynamicName, dynamicData);
        });
    }

    function blockUpgradeUI() {
        const lvl = dynamicData[0].lvl
        const fixPrice = Math.ceil(fixHarvestList[lvl].QPD * ((dynamicData[0].max_durability - dynamicData[0].durability)));
        const itemTranslation = `item.${fixHarvestList[lvl].id.replace("minecraft:", "")}.name`;

        const interfaceTranslate1 = {
            rawtext: [
                { text: "\n" },
                { translate: "gui.auto_harvest.ui_upgrade.body.1" },
                { text: "\n\n" },
                { translate: "gui.auto_harvest.ui_menu.body.1" }, { text: `§e${dynamicData[0].durability} / ${dynamicData[0].max_durability}§r\n` },
                { translate: "gui.auto_harvest.ui_upgrade.body.2" }, { text: `§e${fixPrice} ` }, { translate: itemTranslation },
                { text: "\n " }
            ]
        };

        const interfaceTranslate2 = {
            rawtext: [
                { text: "\n" },
                { translate: "gui.auto_harvest.ui_upgrade.body.1" },
                { text: "\n " }
            ]
        };

        ui_upgrade.title(textT_title);

        if (durabilityBlock) {
            ui_upgrade.body(interfaceTranslate1);
            ui_upgrade.button({ translate: "gui.auto_harvest.ui_upgrade.button1" }, "textures/items/ender_pearl");
        } else {
            ui_upgrade.body(interfaceTranslate2);
        }

        const upgradeButtons = [
            `gui.auto_harvest.ui_upgrade.buttonUpgrade1`,
            `gui.auto_harvest.ui_upgrade.buttonUpgrade2`,
            `gui.auto_harvest.ui_upgrade.buttonUpgrade3`
        ];

        for (let i = 0; i < upgradeButtons.length; i++) {
            if (dynamicData[0].lvl === i) {
                ui_upgrade.button({ translate: upgradeButtons[i] }, "textures/items/amethyst_shard");
            }
        }

        if (lvl <= 3) {

            ui_upgrade.show(player).then((data) => {
                const selectionButton = data.selection;

                function upgradeTest() {
                    const getItemData = getItemInInventory(player, harvestUpgradeList[lvl].id)
                    if (getItemData.itemID == harvestUpgradeList[lvl].id) {
                        const durabilityMax = Math.ceil(dynamicData[0].max_durability * 2)
                        dynamicData[0].lvl += 1;
                        dynamicData[0].max_durability = durabilityMax;
                        dynamicData[0].durability = durabilityMax;
                        dynamicData[0].max_size = harvestUpgradeList[lvl].size
                        dynamicData[0].size = harvestUpgradeList[lvl].size;
                        Dimension.setBlockPermutation(block.location, block.permutation.withState("ext:harvest_states", harvestUpgradeList[lvl].permutation));
                        Dimension.runCommand(`clear "${player.name}" ${harvestUpgradeList[lvl].id} 0 ${harvestUpgradeList[lvl].quantity}`);
                        Dimension.playSound("random.levelup", player.location);
                        VFX(event.block, "minecraft:villager_happy", 1.2, 50, 0.5);
                        writeDynamicData(dynamicName, dynamicData);
                    }
                }

                if (selectionButton === 0 && dynamicData[0].durability < dynamicData[0].max_durability) {
                    const getItemData = getItemInInventory(player, fixHarvestList[lvl].id)
                    if (getItemData.amount >= fixPrice) {
                        dynamicData[0].durability = dynamicData[0].max_durability;
                        Dimension.runCommand(`clear "${player.name}" ${fixHarvestList[lvl].id} 0 ${fixPrice}`);
                        Dimension.playSound("random.levelup", player.location);
                        VFX(event.block, "minecraft:villager_happy", 1.2, 50, 0.5);
                        writeDynamicData(dynamicName, dynamicData);
                    }
                } else {
                    if (selectionButton === 0) {
                        upgradeTest()
                    }
                }
                if (selectionButton === 1) {
                    upgradeTest()
                }
            });
        }
    }

    function handleMenu() {
        if (!dynamicData[0].private) {
            player.isSneaking ? blockMenuUI() : blockMenuUI();
        } else if (player.name === dynamicData[0].owner) {
            blockMenuUI();
        } else {
            player.sendMessage({ translate: "warning.auto_harvest.private" });
        }
    }

    handleMenu();
}

function getDynamicNameInLocation(vector3, dynamicType = "harvest:") {
    for (const objDynamic of world.getDynamicPropertyIds()) {
        if (objDynamic.startsWith(dynamicType)) {
            const blockInLocation = objDynamic.split(dynamicType)[1];
            if (blockInLocation === JSON.stringify(vector3)) {
                return objDynamic;
            }
        }
    }
    return undefined;
}

function getItemInInventory(target, itemID) {
    const inventory = target.getComponent("inventory");
    for (let slotIndex = 0; slotIndex < inventory.inventorySize; slotIndex++) {
        const item = inventory.container.getItem(slotIndex);
        if (item.typeId == itemID) {
            const values = {
                "itemID": item.typeId,
                "amount": item.amount,
                "slotIndex": slotIndex
            };
            return values;
        }
    }
    return undefined;
}


function squareGetBlocks(blockInput, targetList, rad = 3, heightMin = 0, heightMax = 1) {
    try {

        let dynamicName = getDynamicNameInLocation(blockInput.location)
        let dynamicData = fetchDynamicData(dynamicName)

        if (dynamicData[0].durability > 0) {

            const [minX, minY, minZ] = [
                Math.round(blockInput.location.x - rad),
                Math.round(blockInput.location.y - heightMin),
                Math.round(blockInput.location.z - rad)
            ];
            const [maxX, maxY, maxZ] = [
                Math.round(blockInput.location.x + rad),
                Math.round(blockInput.location.y + heightMax),
                Math.round(blockInput.location.z + rad)
            ];

            for (let x = minX; x <= maxX; x++) {
                for (let y = minY; y <= maxY; y++) {
                    for (let z = minZ; z <= maxZ; z++) {
                        const locationVector3 = { x: x, y: y, z: z }
                        const Dimension = world.getDimension(blockInput.dimension.id)
                        const blockInLoc = Dimension.getBlock(locationVector3);
                        if (blockInLoc.typeId !== "minecraft:air") {
                            blocksCollectList.forEach(blocks => {
                                if (blocks.id == blockInLoc.typeId && blockInLoc.typeId !== "minecraft:air") {
                                    if (blockInLoc.permutation.getState("growth") == blocks.data) {
                                        destroyBlock()
                                    }
                                    if (blockInLoc.permutation.getState("growth") == undefined) {
                                        destroyBlock(false)
                                    }
                                    function destroyBlock(placeSeed = true) {
                                        Dimension.runCommand(`setblock ${locationVector3.x} ${locationVector3.y} ${locationVector3.z} air destroy`)
                                        const itemDrop = Dimension.getEntities({ type: "minecraft:item", maxDistance: rad / 2, location: locationVector3 })
                                        for (let yLoc = blockInput.location.y; yLoc < 321; yLoc++) {
                                            if (Dimension.getBlock({ x: blockInput.location.x, y: yLoc, z: blockInput.location.z }).typeId == "minecraft:air") {
                                                itemDrop.forEach(entity => {
                                                    entity.tryTeleport({ x: blockInput.location.x + 0.5, y: yLoc, z: blockInput.location.z + 0.5 })
                                                })
                                                break
                                            }
                                        }
                                        if (placeSeed == true) Dimension.setBlockType(locationVector3, blocks.id);
                                        dynamicData[0].durability = Math.max(dynamicData[0].durability - 1, 0);
                                        writeDynamicData(dynamicName, dynamicData);
                                    }
                                }
                            })
                        }
                    }
                }
            }
        }
    } catch { }
}

const fixHarvestList = [
    {
        "id": "minecraft:copper_ingot",
        "QPD": 0.045
    },
    {
        "id": "minecraft:iron_ingot",
        "QPD": 0.03
    },
    {
        "id": "minecraft:emerald",
        "QPD": 0.02
    },
    {
        "id": "minecraft:diamond",
        "QPD": 0.01
    }
]

const harvestUpgradeList = [
    {
        "id": "minecraft:gold_ingot",
        "quantity": 16,
        "permutation": 3,
        "size": 5
    },
    {
        "id": "minecraft:emerald",
        "quantity": 16,
        "permutation": 4,
        "size": 7
    },
    {
        "id": "minecraft:diamond",
        "quantity": 16,
        "permutation": 5,
        "size": 10
    }
]

const blocksCollectList = [
    {
        "id": "minecraft:wheat",
        "data": 7
    },
    {
        "id": "minecraft:potatoes",
        "data": 7
    },
    {
        "id": "minecraft:beetroot",
        "data": 7
    },
    {
        "id": "minecraft:sweet_berry_bush",
        "data": 3
    },
    {
        "id": "minecraft:carrots",
        "data": 7
    },
    {
        "id": "minecraft:melon_block",
        "data": "minecraft:air"
    },
    {
        "id": "minecraft:pumpkin",
        "data": "minecraft:air"
    }
]