import { world } from "@minecraft/server";

// Funções sendo aprimoradas para a addon do "Herobrine Lurking" e "Extensus".

export function VFX(target, particleID, size, maxParticles = 100, yFIX = 0) {
	const particles = Math.min(maxParticles, 360 / size);
	for (let i = 0; i < particles; i++) {

		const theta = Math.random() * 2 * Math.PI;
		const phi = Math.acos(2 * Math.random() - 1);

		const randomRadius = size + (Math.random() - 0.5) * size * 0.5;

		const xr = (target.location.x + 0.5) + randomRadius * Math.sin(phi) * Math.cos(theta);
		const yr = target.location.y + randomRadius * Math.cos(phi) + yFIX;
		const zr = (target.location.z + 0.5) + randomRadius * Math.sin(phi) * Math.sin(theta);

		const boundedY = Math.max(-64, Math.min(yr, 320));

		const dimension = world.getDimension(target.dimension.id);
		const location = { x: xr, y: boundedY, z: zr };
		dimension.spawnParticle(particleID, location);
	}
}

export const harvestDynamicStructure = (playerPlace) => [
    {

        "owner": playerPlace.name,
        "switch": false,
        "private": false,
        "lvl": 0,

        // Durabilidade do Bloco
        "max_durability": 512,
        "durability": 512,
        
        // Tamanho da Area
        "size": 3,
        "max_size": 3
    }
]

export const itemTeleporterStructure = (playerPlace, blockInput) => [
    {
        "owner": playerPlace.name,
        "switch": false,
        "private": false,
        "intX": blockInput.location.x,
        "intY": blockInput.location.y,
        "intZ": blockInput.location.z
    }
]

export const findIndexPerString = (list, input = "test") => list.findIndex(obj => { return input.toLocaleLowerCase() == obj.name.toLocaleLowerCase() })

export function getDimensionEXT(event = world.getAllPlayers()[0]) {
    try {
        const dimension = world.getDimension(event.dimension.id)
        return dimension
    } catch {
        return world.getDimension("minecraft:overworld")
    }
}

export function randomNumber(min, max, round = true) {
    if (min > max) {
        return null;
    }

    let randomValue = Math.random() * (max - min) + min;

    if (round) {
        randomValue = Math.round(randomValue);
    }

    return randomValue;
}

export function fetchDynamicData(name) {
    try {
        return JSON.parse(world.getDynamicProperty(name))
    } catch {
        console.warn("Erro na conversão, pode ser que esse Dynamic não exista.", name)
        return null
    }
}

export function writeDynamicData(name, input = [{}]) {
    const inputFormat = JSON.stringify(input)
    world.setDynamicProperty(name, inputFormat)
}

export function updateDynamicData(dynamicName, structureConfig = null) {
    try {

        const dynamicData = fetchDynamicData(dynamicName);

        if (dynamicData === null || dynamicData === undefined) {
            writeDynamicData(dynamicName, structureConfig);
        }

        return dynamicData

    } catch (error) {
        writeDynamicData(dynamicName, structureConfig);
        return fetchDynamicData(dynamicName);
    }
}